using System;
using System.Collections;
using System.Text;
using Util;

namespace SkeletalAnimation {

	public struct KeyFrame {
		public uint frame;
		public Quaternion rotation;
	}

	public struct AnimationTrack {
		public uint jointId;
		public ArrayList keyFrames;
	}

	public class Animation {

		#region Member Variables

		private uint numFrames;
		private uint currentFrame;
		
		private ArrayList tracks;

		#endregion

		#region Getters and Setters

		public void setCurrentFrame(uint current) { currentFrame = current; }
		public uint getCurrentFrame() { return currentFrame; }

		public void setNumFrames(uint num) { numFrames = num; }
		public uint getNumFrames() { return numFrames; }

		public ArrayList getTracks() { return tracks; }

		#endregion

		#region Constructors

		public Animation() {
			tracks = new ArrayList();
			currentFrame = 0;
		}

		#endregion

		#region Methods

		public void addAnimationTrack(AnimationTrack track) {
			tracks.Add(track);
		}

		public int getNumTracks() {
			return tracks.Count;
		}

		public AnimationTrack getTrack(int index) {
			return (AnimationTrack)tracks[index];
		}

		public bool isKeyFrame(int trackIndex, uint frame) {
			AnimationTrack track = (AnimationTrack)tracks[trackIndex];
			for(int i=0; i<track.keyFrames.Count && frame >= ((KeyFrame)track.keyFrames[i]).frame; i++) {
				if(frame == ((KeyFrame)track.keyFrames[i]).frame) {
					return true;
				}
			}
			return false;
		}

		public KeyFrame getKeyFrame(int trackIndex, uint frame) {
			AnimationTrack track = (AnimationTrack)tracks[trackIndex];
			KeyFrame key = new KeyFrame();
			for(int i=0; i<track.keyFrames.Count; i++) {
				if(frame == ((KeyFrame)track.keyFrames[i]).frame) {
					key = (KeyFrame)track.keyFrames[i];
					break;
				}
			}
			return key;
		}

		public KeyFrame getNextKeyFrame(int trackIndex, uint frame) {
			AnimationTrack track = (AnimationTrack)tracks[trackIndex];
			KeyFrame key = (KeyFrame)track.keyFrames[0];
			for(int i=0; i<track.keyFrames.Count; i++) {
				if(frame < ((KeyFrame)track.keyFrames[i]).frame) {
					key = (KeyFrame)track.keyFrames[i];
					break;
				}
			}
			return key;
		}

		public KeyFrame getPrevKeyFrame(int trackIndex, uint frame) {
			AnimationTrack track = (AnimationTrack)tracks[trackIndex];
			KeyFrame key = (KeyFrame)track.keyFrames[track.keyFrames.Count-1];
			for(int i=0; i<track.keyFrames.Count; i++) {
				if(frame < ((KeyFrame)track.keyFrames[i]).frame) {
					if(i != 0) {
						key = (KeyFrame)track.keyFrames[i-1];
					}
					break;
				}
			}
			return key;
		}

		#endregion

	}
}
