using System;
using System.Collections;
using System.Text;
using Util;

namespace SkeletalAnimation {
	
	public class Joint {

		#region Member Variables

		private Skeleton skeleton;

		private JointData jointData;

		private int parentJoint;

		private Vector relativeTranslation;
		private Vector absoluteTranslation;

		private Quaternion relativeRotation;
		private Quaternion absoluteRotation;

		#endregion

		#region Constructors

		/**
		 * Default constructor
		 */
		public Joint() {
			skeleton = null;
			jointData = null;
			parentJoint = -1;
			relativeTranslation = null;
			relativeRotation = null;
			absoluteTranslation = null;
			absoluteRotation = null;
		}

		/**
		 * Copy constructor
		 */
		public Joint(JointData jd) {
			skeleton = null;
			jointData = jd;
			parentJoint = jd.getParentJoint();
			relativeTranslation = jd.getRelativeTranslation();
			relativeRotation = jd.getRelativeRotation();
			absoluteTranslation = jd.getAbsoluteTranslation();
			absoluteRotation = jd.getAbsoluteRotation();
		}

		#endregion

		#region Getters And Setters

		public void setSkeleton(Skeleton skel) { this.skeleton = skel; }
		public Skeleton getSkeleton() { return this.skeleton; }

		public void setJointData(JointData joint) { this.jointData = joint; }
		public JointData getJointData() { return this.jointData; }

		public int getParentJoint() { return parentJoint; }
		public void setParentJoint(int parent) { parentJoint = parent; }

		public Quaternion getRelativeRotation() { return relativeRotation; }
		public void setRelativeRotation(Quaternion relRot) { relativeRotation = relRot; }

		public Vector getRelativeTranslation() { return relativeTranslation; }
		public void setRelativeTranslation(Vector relTrans) { relativeTranslation = relTrans; }

		public Quaternion getAbsoluteRotation() { return absoluteRotation; }
		public Vector getAbsoluteTranslation() { return absoluteTranslation; }

		#endregion

		#region Methods

		/**
		 * Return the absolute transformation matrix
		 */
		public Matrix getAbsoluteTransformation() {
			
			Matrix rot = absoluteRotation.toMatrix();
			
			Matrix trans = new Matrix();
			trans.loadIdentity();
			trans[12] = absoluteTranslation[0];
			trans[13] = absoluteTranslation[1];
			trans[14] = absoluteTranslation[2];
			
			return trans * rot;
		}

		/**
		 * Update the absolute rotation an translation
		 */
		public void updateAbsolutes() {
			if(parentJoint != -1) {
				Joint parent = skeleton.getJoint((uint)parentJoint);
				absoluteRotation = parent.absoluteRotation * jointData.getOrientation() * relativeRotation;
				absoluteTranslation = parent.absoluteTranslation + relativeTranslation.rotate(parent.absoluteRotation);
			}
			else {
				absoluteRotation = new Quaternion(jointData.getOrientation() * relativeRotation);
				absoluteTranslation = new Vector(relativeTranslation);
			}
		}

		#endregion
	}
}
