using System;
using System.Collections.Generic;
using System.Text;
using Util;

namespace SkeletalAnimation {
	
	public class JointData {

		#region Member Variables

		private SkeletonData skeletonData;

		private int parentJoint;

		private string name;

		private Quaternion orientation;
	
		private Vector relativeTranslation;
		private Vector absoluteTranslation;

		private Quaternion relativeRotation;
		private Quaternion absoluteRotation;

		#endregion

		#region Getters And Setters

		// Getters and setters

		public SkeletonData getSkeletonData() { return skeletonData; }
		public void setSkeletonData(SkeletonData skelData) { this.skeletonData = skelData; }

		public int getParentJoint() { return parentJoint; }
		public void setParentJoint(int parent) { parentJoint = parent; }

		public string getName() { return name; }
		public void setName(string nm) { name = nm; }

		public Quaternion getOrientation() { return orientation; }
		public void setOrientation(Quaternion or) { orientation = or; }

		public Quaternion getRelativeRotation() { return relativeRotation; }
		public void setRelativeRotation(Quaternion relRot) { relativeRotation = relRot; }

		public Vector getRelativeTranslation() { return relativeTranslation; }
		public void setRelativeTranslation(Vector relTrans) { relativeTranslation = relTrans; }

		public Quaternion getAbsoluteRotation() { return absoluteRotation; }
		public Vector getAbsoluteTranslation() { return absoluteTranslation; }

		#endregion

		#region Constructors

		/**
		 * Default constructor
		 */
		public JointData() {
			skeletonData = null;
			parentJoint = -1;
			relativeTranslation = null;
			relativeRotation = null;
			absoluteTranslation = null;
			absoluteRotation = null;
		}

		/**
		 * Copy Constructor
		 */
		public JointData(JointData jointData) {
			skeletonData = jointData.skeletonData;
			parentJoint = jointData.parentJoint;
			name = jointData.name;
			orientation = jointData.orientation;
			relativeTranslation = jointData.relativeTranslation;
			relativeRotation = jointData.relativeRotation;
			absoluteTranslation = jointData.absoluteTranslation;
			absoluteRotation = jointData.absoluteRotation;
		}

		#endregion

		#region Methods

		/**
		 * Return the inverse absolute transformation matrix
		 */
		public Matrix getAbsoluteTransformationInverse() {
			
			Matrix rot = absoluteRotation.toMatrix().getTransposed();

			Vector temp = rot * absoluteTranslation;
			Matrix trans = new Matrix();
			trans.loadIdentity();
			trans[12] = -temp[0];
			trans[13] = -temp[1];
			trans[14] = -temp[2];

			return trans * rot;
		}

		/**
		 * Calculate the absolute rotation and translation
		 */
		public void updateAbsolutes() {
			if(parentJoint != -1) {
				JointData parent = skeletonData.getJointData(parentJoint);
				absoluteRotation = parent.absoluteRotation * orientation * relativeRotation;
				absoluteTranslation = parent.getAbsoluteTranslation() + relativeTranslation.rotate(parent.absoluteRotation);
			}
			else {
				absoluteRotation = new Quaternion(orientation * relativeRotation);
				absoluteTranslation = new Vector(relativeTranslation);
			}
		}

		#endregion
	}
}
