using System;
using System.Collections.Generic;
using System.Text;

namespace SkeletalAnimation {
	
	public class Matrix {

		double[] values = new double[16];

		/**
		 * Constructor. Loads the identity matrix
		 */
		public Matrix() {
			values[0] = 1; values[4] = 0; values[8] = 0; values[12] = 0;
			values[1] = 0; values[5] = 1; values[9] = 0; values[13] = 0;
			values[2] = 0; values[6] = 0; values[10] = 1; values[14] = 0;
			values[3] = 0; values[7] = 0; values[11] = 0; values[15] = 1;
		}


		/**
		 * Indexer
		 */
		public double this[int index] {
			get {
				return values[index];
			}
			set {
				values[index] = value;
			}
		}	

		/**
		 * Overloading the multiplication operator
		 */
		public static Matrix operator *(Matrix left, Matrix right) {
			Matrix ret = new Matrix();

			// Row 1
			ret[0] = left[0] * right[0] + left[4] * right[1] + left[8] * right[2] + left[12] * right[3];
			ret[4] = left[0] * right[4] + left[4] * right[5] + left[8] * right[6] + left[12] * right[7];
			ret[8] = left[0] * right[8] + left[4] * right[9] + left[8] * right[10] + left[12] * right[11];
			ret[12] = left[0] * right[12] + left[4] * right[13] + left[8] * right[14] + left[12] * right[15];
			// Row 2
			ret[1] = left[1] * right[0] + left[5] * right[1] + left[9] * right[2] + left[13] * right[3];
			ret[5] = left[1] * right[4] + left[5] * right[5] + left[9] * right[6] + left[13] * right[7];
			ret[9] = left[1] * right[8] + left[5] * right[9] + left[9] * right[10] + left[13] * right[11];
			ret[13] = left[1] * right[12] + left[5] * right[13] + left[9] * right[14] + left[13] * right[15];
			// Row 3
			ret[2] = left[2] * right[0] + left[6] * right[1] + left[10] * right[2] + left[14] * right[3];
			ret[6] = left[2] * right[4] + left[6] * right[5] + left[10] * right[6] + left[14] * right[7];
			ret[10] = left[2] * right[8] + left[6] * right[9] + left[10] * right[10] + left[14] * right[11];
			ret[14] = left[2] * right[12] + left[6] * right[13] + left[10] * right[14] + left[14] * right[15];
			// Row 4
			ret[3] = left[3] * right[0] + left[7] * right[1] + left[11] * right[2] + left[15] * right[3];
			ret[7] = left[3] * right[4] + left[7] * right[5] + left[11] * right[6] + left[15] * right[7];
			ret[11] = left[3] * right[8] + left[7] * right[9] + left[11] * right[10] + left[15] * right[11];
			ret[15] = left[3] * right[12] + left[7] * right[13] + left[11] * right[14] + left[15] * right[15];

			return ret;
		}

		/**
		 * Return identity matrix
		 */
		public static Matrix getIdentity() {
			Matrix ret = new Matrix();
			return ret;
		}
	}
}
