using System;
using System.Collections;
using System.Text;

namespace SkeletalAnimation {
	
	/**
	 * Bone Influence structure. This structure is uses to keep track 
	 * of which bones is affecting each vertex
	 */
	public struct BoneInfluence {
		public uint boneId;
		public double weight;
	}

	/**
	 * This class holds the default mesh ie. Bind Pose
	 */
	public class MeshData {

		#region Member Variables

		private String name;

		private SkeletonData skeletonData;

		private ArrayList[] influences;

		private double[] vertices;
		private uint[] vertexIndices;
		
		private double[] normals;
		// Array for relating a normal to a vertex. This helps the calculation 
		// of the normals after transform, but I don't like the solution.
		private uint[] normalToVertex;	
		private uint[] normalIndices;

		private double[] uvs;
		private uint[] uvIndices;

		#endregion

		#region Getters & Setters

		// Getters & Setters

		public void setName(String n) { name = n; }
		public String getName() { return name; }

		public void setSkeletonData(SkeletonData skel) { skeletonData = skel; }
		public SkeletonData getSkeletonData() { return skeletonData; }

		public void setInfluences(ArrayList[] infl) { influences = infl; }
		public ArrayList[] getInfluences() { return influences; }

		public void setVertices(double[] vert) { vertices = vert; }
		public double[] getVertices() { return vertices; }
		public void setVertexIndices(uint[] indices) { vertexIndices = indices; }
		public uint[] getVertexIndices() { return vertexIndices; }

		public void setNormals(double[] norm) { normals = norm; }
		public double[] getNormals() { return normals; }
		public void setNormalToVertex(uint[] norm) { normalToVertex = norm; }
		public uint[] getNormalToVertex() { return normalToVertex; }
		public void setNormalIndices(uint[] indices) { normalIndices = indices; }
		public uint[] getNormalIndices() { return normalIndices; }

		public void setUVs(double[] uv) { uvs = uv; }
		public double[] getUVs() { return uvs; }
		public void setUVIndices(uint[] indices) { uvIndices = indices; }
		public uint[] getUVIndices() { return uvIndices; }

		//public uint[] getIndices() { return indices; }

		#endregion

		#region Constructors

		/**
		 * Default constructor
		 */
		public MeshData() {
			skeletonData = new SkeletonData();

			/*
			// Influences
			influences = new ArrayList[32];
			BoneInfluence boneInfluence0 = new BoneInfluence();
			boneInfluence0.boneId = 0;
			boneInfluence0.weight = 1.0;
			influences[0] = new ArrayList();
			influences[0].Add(boneInfluence0);

			BoneInfluence boneInfluence1 = new BoneInfluence();
			boneInfluence1.boneId = 0;
			boneInfluence1.weight = 1.0;
			influences[1] = new ArrayList();
			influences[1].Add(boneInfluence1);

			BoneInfluence boneInfluence2 = new BoneInfluence();
			boneInfluence2.boneId = 0;
			boneInfluence2.weight = 1.0;
			influences[2] = new ArrayList();
			influences[2].Add(boneInfluence2);

			BoneInfluence boneInfluence3 = new BoneInfluence();
			boneInfluence3.boneId = 0;
			boneInfluence3.weight = 1.0;
			influences[3] = new ArrayList();
			influences[3].Add(boneInfluence3);

			BoneInfluence boneInfluence4 = new BoneInfluence();
			boneInfluence4.boneId = 0;
			boneInfluence4.weight = 1.0;
			influences[4] = new ArrayList();
			influences[4].Add(boneInfluence4);

			BoneInfluence boneInfluence5 = new BoneInfluence();
			boneInfluence5.boneId = 0;
			boneInfluence5.weight = 1.0;
			influences[5] = new ArrayList();
			influences[5].Add(boneInfluence5);

			BoneInfluence boneInfluence6 = new BoneInfluence();
			boneInfluence6.boneId = 0;
			boneInfluence6.weight = 1.0;
			influences[6] = new ArrayList();
			influences[6].Add(boneInfluence6);

			BoneInfluence boneInfluence7 = new BoneInfluence();
			boneInfluence7.boneId = 0;
			boneInfluence7.weight = 1.0;
			influences[7] = new ArrayList();
			influences[7].Add(boneInfluence7);

			BoneInfluence boneInfluence8 = new BoneInfluence();
			boneInfluence8.boneId = 0;
			boneInfluence8.weight = 1.0;
			influences[8] = new ArrayList();
			influences[8].Add(boneInfluence8);

			BoneInfluence boneInfluence9 = new BoneInfluence();
			boneInfluence9.boneId = 0;
			boneInfluence9.weight = 1.0;
			influences[9] = new ArrayList();
			influences[9].Add(boneInfluence9);

			BoneInfluence boneInfluence10 = new BoneInfluence();
			boneInfluence10.boneId = 0;
			boneInfluence10.weight = 1.0;
			influences[10] = new ArrayList();
			influences[10].Add(boneInfluence10);

			BoneInfluence boneInfluence11 = new BoneInfluence();
			boneInfluence11.boneId = 0;
			boneInfluence11.weight = 1.0;
			influences[11] = new ArrayList();
			influences[11].Add(boneInfluence11);

			BoneInfluence boneInfluence12 = new BoneInfluence();
			boneInfluence12.boneId = 1;
			boneInfluence12.weight = 1.0;
			influences[12] = new ArrayList();
			influences[12].Add(boneInfluence12);

			BoneInfluence boneInfluence13 = new BoneInfluence();
			boneInfluence13.boneId = 1;
			boneInfluence13.weight = 1.0;
			influences[13] = new ArrayList();
			influences[13].Add(boneInfluence13);

			BoneInfluence boneInfluence14 = new BoneInfluence();
			boneInfluence14.boneId = 1;
			boneInfluence14.weight = 1.0;
			influences[14] = new ArrayList();
			influences[14].Add(boneInfluence14);

			BoneInfluence boneInfluence15 = new BoneInfluence();
			boneInfluence15.boneId = 1;
			boneInfluence15.weight = 1.0;
			influences[15] = new ArrayList();
			influences[15].Add(boneInfluence15);

			BoneInfluence boneInfluence16 = new BoneInfluence();
			boneInfluence16.boneId = 1;
			boneInfluence16.weight = 1.0;
			influences[16] = new ArrayList();
			influences[16].Add(boneInfluence16);

			BoneInfluence boneInfluence17 = new BoneInfluence();
			boneInfluence17.boneId = 1;
			boneInfluence17.weight = 1.0;
			influences[17] = new ArrayList();
			influences[17].Add(boneInfluence17);

			BoneInfluence boneInfluence18 = new BoneInfluence();
			boneInfluence18.boneId = 1;
			boneInfluence18.weight = 1.0;
			influences[18] = new ArrayList();
			influences[18].Add(boneInfluence18);

			BoneInfluence boneInfluence19 = new BoneInfluence();
			boneInfluence19.boneId = 1;
			boneInfluence19.weight = 1.0;
			influences[19] = new ArrayList();
			influences[19].Add(boneInfluence19);

			BoneInfluence boneInfluence20 = new BoneInfluence();
			boneInfluence20.boneId = 2;
			boneInfluence20.weight = 1.0;
			influences[20] = new ArrayList();
			influences[20].Add(boneInfluence20);

			BoneInfluence boneInfluence21 = new BoneInfluence();
			boneInfluence21.boneId = 2;
			boneInfluence21.weight = 1.0;
			influences[21] = new ArrayList();
			influences[21].Add(boneInfluence21);

			BoneInfluence boneInfluence22 = new BoneInfluence();
			boneInfluence22.boneId = 2;
			boneInfluence22.weight = 1.0;
			influences[22] = new ArrayList();
			influences[22].Add(boneInfluence22);

			BoneInfluence boneInfluence23 = new BoneInfluence();
			boneInfluence23.boneId = 2;
			boneInfluence23.weight = 1.0;
			influences[23] = new ArrayList();
			influences[23].Add(boneInfluence23);

			BoneInfluence boneInfluence24 = new BoneInfluence();
			boneInfluence24.boneId = 2;
			boneInfluence24.weight = 1.0;
			influences[24] = new ArrayList();
			influences[24].Add(boneInfluence24);

			BoneInfluence boneInfluence25 = new BoneInfluence();
			boneInfluence25.boneId = 2;
			boneInfluence25.weight = 1.0;
			influences[25] = new ArrayList();
			influences[25].Add(boneInfluence25);

			BoneInfluence boneInfluence26 = new BoneInfluence();
			boneInfluence26.boneId = 2;
			boneInfluence26.weight = 1.0;
			influences[26] = new ArrayList();
			influences[26].Add(boneInfluence26);

			BoneInfluence boneInfluence27 = new BoneInfluence();
			boneInfluence27.boneId = 2;
			boneInfluence27.weight = 1.0;
			influences[27] = new ArrayList();
			influences[27].Add(boneInfluence27);

			BoneInfluence boneInfluence28 = new BoneInfluence();
			boneInfluence28.boneId = 2;
			boneInfluence28.weight = 1.0;
			influences[28] = new ArrayList();
			influences[28].Add(boneInfluence28);

			BoneInfluence boneInfluence29 = new BoneInfluence();
			boneInfluence29.boneId = 2;
			boneInfluence29.weight = 1.0;
			influences[29] = new ArrayList();
			influences[29].Add(boneInfluence29);

			BoneInfluence boneInfluence30 = new BoneInfluence();
			boneInfluence30.boneId = 2;
			boneInfluence30.weight = 1.0;
			influences[30] = new ArrayList();
			influences[30].Add(boneInfluence30);

			BoneInfluence boneInfluence31 = new BoneInfluence();
			boneInfluence31.boneId = 2;
			boneInfluence31.weight = 1.0;
			influences[31] = new ArrayList();
			influences[31].Add(boneInfluence31);

			//Vertices
			vertices = new double[96];
			vertices[0] = -1.0; vertices[1] = 0.0; vertices[2] = 1.0;		// 0
			vertices[3] = 1.0; vertices[4] = 0.0; vertices[5] = 1.0;		// 1
			vertices[6] = 1.0; vertices[7] = 0.0; vertices[8] = -1.0;		// 2
			vertices[9] = -1.0; vertices[10] = 0.0; vertices[11] = -1.0;// 3
			
			vertices[12] = -1.0; vertices[13] = 0.0; vertices[14] = 1.0;
			vertices[15] = 1.0; vertices[16] = 0.0; vertices[17] = 1.0;
			vertices[18] = 1.0; vertices[19] = 0.0; vertices[20] = -1.0;
			vertices[21] = -1.0; vertices[22] = 0.0; vertices[23] = -1.0;
			
			vertices[24] = -1.0; vertices[25] = 0.0; vertices[26] = 1.0;
			vertices[27] = 1.0; vertices[28] = 0.0; vertices[29] = 1.0;
			vertices[30] = 1.0; vertices[31] = 0.0; vertices[32] = -1.0;
			vertices[33] = -1.0; vertices[34] = 0.0; vertices[35] = -1.0;

			vertices[36] = -1.0; vertices[37] = 1.0; vertices[38] = 1.0;
			vertices[39] = 1.0; vertices[40] = 1.0; vertices[41] = 1.0;
			vertices[42] = 1.0; vertices[43] = 1.0; vertices[44] = -1.0;
			vertices[45] = -1.0; vertices[46] = 1.0; vertices[47] = -1.0;

			vertices[48] = -1.0; vertices[49] = 1.0; vertices[50] = 1.0;
			vertices[51] = 1.0; vertices[52] = 1.0; vertices[53] = 1.0;
			vertices[54] = 1.0; vertices[55] = 1.0; vertices[56] = -1.0;
			vertices[57] = -1.0; vertices[58] = 1.0; vertices[59] = -1.0;

			vertices[60] = -1.0; vertices[61] = 2.0; vertices[62] = 1.0;
			vertices[63] = 1.0; vertices[64] = 2.0; vertices[65] = 1.0;
			vertices[66] = 1.0; vertices[67] = 2.0; vertices[68] = -1.0;
			vertices[69] = -1.0; vertices[70] = 2.0; vertices[71] = -1.0;

			vertices[72] = -1.0; vertices[73] = 2.0; vertices[74] = 1.0;
			vertices[75] = 1.0; vertices[76] = 2.0; vertices[77] = 1.0;
			vertices[78] = 1.0; vertices[79] = 2.0; vertices[80] = -1.0;
			vertices[81] = -1.0; vertices[82] = 2.0; vertices[83] = -1.0;

			vertices[84] = -1.0; vertices[85] = 2.0; vertices[86] = 1.0;
			vertices[87] = 1.0; vertices[88] = 2.0; vertices[89] = 1.0;
			vertices[90] = 1.0; vertices[91] = 2.0; vertices[92] = -1.0;
			vertices[93] = -1.0; vertices[94] = 2.0; vertices[95] = -1.0;

			// Normals
			normals = new double[96];
			normals[0] = 0.0; normals[1] = -1.0; normals[2] = 0.0;
			normals[3] = 0.0; normals[4] = -1.0; normals[5] = 0.0;
			normals[6] = 0.0; normals[7] = -1.0; normals[8] = 0.0;
			normals[9] = 0.0; normals[10] = -1.0; normals[11] = 0.0;

			normals[12] = -1.0; normals[13] = 0.0; normals[14] = 0.0;
			normals[15] = 1.0; normals[16] = 0.0; normals[17] = 0.0;
			normals[18] = 1.0; normals[19] = 0.0; normals[20] = 0.0;
			normals[21] = -1.0; normals[22] = 0.0; normals[23] = 0.0;

			normals[24] = 0.0; normals[25] = 0.0; normals[26] = 1.0;
			normals[27] = 0.0; normals[28] = 0.0; normals[29] = 1.0;
			normals[30] = 0.0; normals[31] = 0.0; normals[32] = -1.0;
			normals[33] = 0.0; normals[34] = 0.0; normals[35] = -1.0;

			normals[36] = -1.0; normals[37] = 0.0; normals[38] = 0.0;
			normals[39] = 1.0; normals[40] = 0.0; normals[41] = 0.0;
			normals[42] = 1.0; normals[43] = 0.0; normals[44] = 0.0;
			normals[45] = -1.0; normals[46] = 0.0; normals[47] = 0.0;

			normals[48] = 0.0; normals[49] = 0.0; normals[50] = 1.0;
			normals[51] = 0.0; normals[52] = 0.0; normals[53] = 1.0;
			normals[54] = 0.0; normals[55] = 0.0; normals[56] = -1.0;
			normals[57] = 0.0; normals[58] = 0.0; normals[59] = -1.0;

			normals[60] = -1.0; normals[61] = 0.0; normals[62] = 0.0;
			normals[63] = 1.0; normals[64] = 0.0; normals[65] = 0.0;
			normals[66] = 1.0; normals[67] = 0.0; normals[68] = 0.0;
			normals[69] = -1.0; normals[70] = 0.0; normals[71] = 0.0;

			normals[72] = 0.0; normals[73] = 0.0; normals[74] = 1.0;
			normals[75] = 0.0; normals[76] = 0.0; normals[77] = 1.0;
			normals[78] = 0.0; normals[79] = 0.0; normals[80] = -1.0;
			normals[81] = 0.0; normals[82] = 0.0; normals[83] = -1.0;

			normals[84] = 0.0; normals[85] = 1.0; normals[86] = 0.0;
			normals[87] = 0.0; normals[88] = 1.0; normals[89] = 0.0;
			normals[90] = 0.0; normals[91] = 1.0; normals[92] = 0.0;
			normals[93] = 0.0; normals[94] = 1.0; normals[95] = 0.0;

			uvs = new double[24];

			indices = new uint[60];
			// Bottom
			indices[0] = 3; indices[1] = 2; indices[2] = 0;
			indices[3] = 2; indices[4] = 1; indices[5] = 0;
			
			// Left
			indices[6] = 7; indices[7] = 4; indices[8] = 15;
			indices[9] = 4; indices[10] = 12; indices[11] = 15;

			//Right
			indices[12] = 5; indices[13] = 6; indices[14] = 13;
			indices[15] = 6; indices[16] = 14; indices[17] = 13;

			//Front
			indices[18] = 8; indices[19] = 9; indices[20] = 16;
			indices[21] = 9; indices[22] = 17; indices[23] = 16;
			
			// Back
			indices[24] = 10; indices[25] = 11; indices[26] = 18;
			indices[27] = 11; indices[28] = 19; indices[29] = 18;

			// Left
			indices[30] = 15; indices[31] = 12; indices[32] = 23;
			indices[33] = 12; indices[34] = 20; indices[35] = 23;

			// Right
			indices[36] = 13; indices[37] = 14; indices[38] = 21;
			indices[39] = 14; indices[40] = 22; indices[41] = 21;

			//Front
			indices[42] = 16; indices[43] = 17; indices[44] = 24;
			indices[45] = 17; indices[46] = 25; indices[47] = 24;

			// Back
			indices[48] = 18; indices[49] = 19; indices[50] = 26;
			indices[51] = 19; indices[52] = 27; indices[53] = 26;

			// Top
			indices[54] = 28; indices[55] = 29; indices[56] = 31;
			indices[57] = 29; indices[58] = 30; indices[59] = 31;
			 */
			
		}

		#endregion

		#region Methods

		/**
		 * Create an instance of the mesh data
		 */
		public Mesh createInstance() {
			Mesh mesh = new Mesh();

			mesh.setMeshData(this);
			mesh.setSkeleton(skeletonData.createInstance());
			mesh.setVertices(new double[vertices.Length]);
			mesh.setNormals(new double[normals.Length]);
			
			mesh.createVertexArrays(vertexIndices.Length);
			mesh.fillInUVArray();

			return mesh;
		}

		public ArrayList getInfluences(uint vertexIndex) {
			return influences[vertexIndex];
		}

		#endregion

		
	}
}
