using System;
using System.Collections.Generic;
using System.Text;

namespace SkeletalAnimation {
	
	public class Quaternion {

		private double[] values;

		/**
		 * Default Constructor
		 */
		public Quaternion() {
			values = new double[4];
		}

		/**
		 * Copy Constructor
		 */
		public Quaternion(Quaternion quaternion) {
			values = new double[4];

			values[0] = quaternion[0];
			values[1] = quaternion[1];
			values[2] = quaternion[2];
			values[3] = quaternion[3];
		}

		/**
		 * Indexer
		 */
		public double this[int index] {
			get {
				return this.values[index];
			}
			set {
				this.values[index] = value;
			}
		}

		/* Not needed and would screw with the rotation
		public static Quaternion operator +(Quaternion left, Quaternion right) {
			Quaternion ret = new Quaternion();

			ret.values[0] = left[0] + right[0];
			ret.values[1] = left[1] + right[1];
			ret.values[2] = left[2] + right[2];
			ret.values[3] = left[3] + right[3];

			return ret;
		}*/

		/**
		 * Multiplication operator
		 */
		public static Quaternion operator *(Quaternion left, Quaternion right) {
			Quaternion ret = new Quaternion();

			ret[0] = left[1]*right[2] - left[2]*right[1] + left[3]*right[0] + left[0]*right[3];
			ret[1] = left[2]*right[0] - left[0]*right[2] + left[3]*right[1] + left[1]*right[3];
			ret[2] = left[0]*right[1] - left[1]*right[0] + left[3]*right[2] + left[2]*right[3];
			ret[3] = left[3]*right[3] - left[0]*right[0] - left[1]*right[1] - left[2]*right[2];

			return ret;
		}

		/**
		 * Method for setting the quaternion from an axis and an angle
		 * @param axis - Vector holding the axis
		 * @param angle - Double holding the angle of rotation
		 */
		public void setQuaternion(Vector axis, double angle) {
			double halfAngle = angle/2;
			double sinAngle = Math.Sin(halfAngle);
			double cosAngle = Math.Cos(halfAngle);

			values[0] = sinAngle * axis[0];
			values[1] = sinAngle * axis[1];
			values[2] = sinAngle * axis[2];
			values[3] = cosAngle;
		}

		/**
		 * Method for setting the quaternion from amatrix
		 * @param matrix - Matrix holding the desired rotation
		 */
		public void setQuaternion(Matrix matrix) {

		}

		/**
		 * Return an inverse quaternion
		 * @remark
		 *		The math is:  q* \ |q| where q* is the conjugate of the quaternion 
		 *		and |q| is the magnitude, but since we only work with unit quaternions
		 *		we can skip the divide. 
		 * @return The conjugate quaternion to this
		 */
		public Quaternion getInverseQuaternion() {
			Quaternion ret = new Quaternion();
			ret[0] = -this[0];
			ret[1] = -this[1];
			ret[2] = -this[2];
			ret[3] = this[3];

			return ret;
		}

		/**
		 * Return a matrix equivalent to the quaternion.
		 * @remark 
		 *	Assumes the quaternion is normalized (x^2 + y^2 + z^2 + w^2 = 1). No
		 *	test is performed to check.
		 */
		public Matrix getMatrix() {
			Matrix ret = new Matrix();
			
			// Row 1
			ret[0] = 1 - 2*(this.values[1]*this.values[1] + this.values[2]*this.values[2]);
			ret[4] = 2*(this.values[0]*this.values[1] - this.values[3]*this.values[2]);
			ret[8] = 2*(this.values[3]*this.values[1] + this.values[0]*this.values[2]);
			ret[12] = 0;

			// Row 2
			ret[1] = 2*(this.values[0]*this.values[1] + this.values[3]*this.values[2]);
			ret[5] = 1 - 2*(this.values[0]*this.values[0] + this.values[2]*this.values[2]);
			ret[9] = 2*(this.values[1]*this.values[2] - this.values[3]*this.values[0]);
			ret[13] = 0;

			// Row 3
			ret[2] = 2*(this.values[0]*this.values[2] - this.values[3]*this.values[1]);
			ret[6] = 2*(this.values[1]*this.values[2] + this.values[3]*this.values[0]);
			ret[10] = 1 - 2*(this.values[0]*this.values[0] + this.values[1]*this.values[1]);
			ret[14] = 0;

			// Row 4
			ret[3] = 0;
			ret[7] = 0;
			ret[11] = 0;
			ret[15] = 1;

			return ret;
		}

		/**
		 * Rotate a vector around this quaternion
		 * @param vector - The vector we wish to rotate
		 */
		public void rotateVector(Vector vector) {
			
			// Create a "pure" quaternion from the vector
			Quaternion qVector = new Quaternion();
			qVector[0] = vector[0];
			qVector[1] = vector[1];
			qVector[2] = vector[2];
			qVector[3] = 0;

			// Rotate the pure quaternion
			Quaternion qResult = this * qVector * this.getInverseQuaternion();

			// Convert back to vector
			vector[0] = qResult[0];
			vector[1] = qResult[1];
			vector[2] = qResult[2];
		}		
	}
}
