using System;
using System.Collections;
using System.Text;
using System.IO;
using System.Globalization;
using Util;

namespace SkeletalAnimation {

	

	public class RAALoader {

		#region RAA Loader

		public Animation readRAA(string filename) {
			StreamReader fileStream = new StreamReader(filename);
			String fileData = fileStream.ReadToEnd();
			fileStream.Close();

			char[] delimiters = new char[4];
			delimiters[0] = ' ';
			delimiters[1] = '\t';
			delimiters[2] = '\r';
			delimiters[3] = '\n';

			String[] substrings = fileData.Split(delimiters, StringSplitOptions.RemoveEmptyEntries);

			// Number format
			NumberFormatInfo numberFormat = new NumberFormatInfo();
			numberFormat.NumberDecimalSeparator = ".";

			Animation animation = new Animation();
			
			int index = 0;

			// Extract keyframes
			if(substrings[index++] == "animation") {
				animation.setNumFrames(UInt32.Parse(substrings[index++]));
				uint numTracks = UInt32.Parse(substrings[index++]);

				for(uint track=0; track<numTracks; track++) {
					
					index++; // Skip track keyword
					index++; // Skip track identifier

					AnimationTrack newTrack = new AnimationTrack();
					newTrack.keyFrames = new ArrayList();

					newTrack.jointId = UInt32.Parse(substrings[index++]);

					index++;	// Skip joint name

					if(substrings[index++] == "keyframes") {

						uint numKeyFrames = UInt32.Parse(substrings[index++]);

						for(uint frame=0; frame<numKeyFrames; frame++) {
							index++;	// Skip frame identifier

							KeyFrame newKeyFrame = new KeyFrame();
							newKeyFrame.frame = UInt32.Parse(substrings[index++]);
							
							Quaternion rotation = new Quaternion(Double.Parse(substrings[index++], numberFormat),
								Double.Parse(substrings[index++], numberFormat),
								Double.Parse(substrings[index++], numberFormat),
								Double.Parse(substrings[index++], numberFormat));
							newKeyFrame.rotation = rotation;
							newTrack.keyFrames.Add(newKeyFrame);
						}
					}			

					animation.addAnimationTrack(newTrack);
				}
			}

			return animation;
		}

		#endregion

	}

	
}
