using System;
using System.Collections.Generic;
using System.Text;

namespace Util
{
	public class Vector {

		#region Member Variables

		private double[] values;

		#endregion

		#region Constructors

		/**
		 * Default Constructor
		 */
		public Vector() {
			values = new double[3];
		}

		/**
		 * Copy Constructor
		 */
		public Vector(Vector vector) {
			values = new double[3];

			values[0] = vector[0];
			values[1] = vector[1];
			values[2] = vector[2];
		}

		/**
		 * Parameterized Constructor
		 */
		public Vector(double x, double y, double z) {
			values = new double[3];

			values[0] = x;
			values[1] = y;
			values[2] = z;
		}

		#endregion

		#region Indexer

		/**
		 * Indexer
		 */
		public double this[int index] {
			get {
				return values[index];
			}
			set {
				values[index] = value;
			}
		}

		#endregion

		#region Operator Overloaders

		/**
		 * Oveloading the addition operator
		 */
		public static Vector operator +(Vector left, Vector right) {
			Vector ret = new Vector();
			ret[0] = left[0] + right[0];
			ret[1] = left[1] + right[1];
			ret[2] = left[2] + right[2];
			return ret;
		}

		#endregion

		#region Methods

		/**
		 * Scalar product. Dot product
		 */
		public double dotProduct(Vector other) {
			return values[0]*other[0] + values[1]*other[1] + values[2]*other[2];
		}

		/**
		 * Cross product of two vectors
		 */
		public Vector crossProduct(Vector other) {
			Vector ret = new Vector();

			ret[0] = values[1]*other[2] - values[2]*other[1];
			ret[1] = values[2]*other[0] - values[0]*other[2];
			ret[2] = values[0]*other[1] - values[1]*other[0];

			return ret;
		}

		/**
		 * Rotate this vector around a quaternion
		 * @param vector - The quaternion we wish to rotate with
		 */
		public Vector rotate(Quaternion quaternion) {

			// Create a "pure" quaternion from the vector
			Quaternion qVector = new Quaternion();
			qVector[0] = values[0];
			qVector[1] = values[1];
			qVector[2] = values[2];
			qVector[3] = 0;

			// Rotate the pure quaternion
			Quaternion qResult = quaternion * qVector * quaternion.getConjugate();

			// Convert back to vector
			Vector ret = new Vector();
			ret[0] = qResult[0];
			ret[1] = qResult[1];
			ret[2] = qResult[2];

			return ret;
		}

		#endregion
	}
}
